@@ -51,6 +51,8 @@
 /* table of KolabMailAccess objects */
 
 static GHashTable *koma_objects = NULL;
+static GMutex *koma_objects_lock = NULL;
+static GMutex *active_cal_views_lock = NULL;
 
 /*----------------------------------------------------------------------------*/
 /* forward declarations */
@@ -88,11 +90,9 @@ typedef struct {
 /* Private part of the ECalBackendKolab structure */
 typedef struct _ECalBackendKolabPrivate ECalBackendKolabPrivate;
 struct _ECalBackendKolabPrivate {
-	GHashTable *koma_table;
 	CalMode cal_mode;
 	KolabMailAccess *cal_koma;
 	ECalBackendCache *cal_cache;
-	GHashTable *active_cal_views;
 	gchar *user_email;
 	ECalComponent *default_zone;
 	ECalSourceType source_type;
@@ -248,6 +248,7 @@ e_cal_backend_kolab_open (ECalBackendSync *backend,
 	gchar *tmp_key = NULL;
 	const gchar *prop_str = NULL;
 	gboolean online = FALSE;
+	gboolean auth_required = FALSE;
 	gboolean ok = FALSE;
 	GError *tmp_err = NULL;
 
@@ -260,6 +261,8 @@ e_cal_backend_kolab_open (ECalBackendSync *backend,
 	self = E_CAL_BACKEND_KOLAB (backend);
 	priv = E_CAL_BACKEND_KOLAB_PRIVATE (self);
 
+	g_mutex_lock (koma_objects_lock);
+
 	/* to trigger eds to pass along username and password, set the property
 	 * "auth" to "true" in the source setting:
 	 * <property name="auth" value="true"/>
@@ -279,23 +282,11 @@ e_cal_backend_kolab_open (ECalBackendSync *backend,
 		priv->source_type = E_CAL_SOURCE_TYPE_JOURNAL;
 		break;
 	default:
-		/* FIXME */
-		g_error ("%s()[%u]: Unknown Type used in e-cal-backend-kolab initialization!",
-		         __func__, __LINE__);
-	}
-
-	kolab_util_glib_init ();
-	kolab_util_http_init ();
-	/* libcamel
-	 * Curl init may configure the underlying SSL lib,
-	 * but as far as SSL goes, we want Camel to rule here
-	 * TODO check whether Camel session needs to be initialized before or after libcurl.
-	 */
-	ok = kolab_util_camel_init (&tmp_err);
-	if (! ok) {
-		kolab_util_calendar_err_to_edb_err (error, tmp_err, __func__, __LINE__);
-		g_error_free (tmp_err);
-		return;
+		g_set_error (&tmp_err,
+		             KOLAB_BACKEND_ERROR,
+		             KOLAB_BACKEND_ERROR_GENERIC,
+		             "Unknown type used in e-cal-backend-kolab initialization");
+		goto exit;
 	}
 
 	esource = e_backend_get_source (E_BACKEND (backend));
@@ -303,25 +294,18 @@ e_cal_backend_kolab_open (ECalBackendSync *backend,
 		/* FIXME mark this as a translatable string */
 		tmp_err = e_data_cal_create_error (OtherError,
 		                                   "Could not get ESource for backend");
-		g_propagate_error (error, tmp_err);
-		return;
+		goto exit;
 	}
 
 	priv->cal_uri = e_source_get_uri (esource);
-
 	c_url = camel_url_new (priv->cal_uri, &tmp_err);
-	if (c_url == NULL) {
-		kolab_util_calendar_err_to_edb_err (error, tmp_err, __func__, __LINE__);
-		g_error_free (tmp_err);
-		return;
-	}
+	if (c_url == NULL)
+		goto exit;
+
 	servername = g_strdup (c_url->host);
 	username = g_strdup (c_url->user);
-	camel_url_free (c_url);
-	g_debug ("%s()[%u] servername = %s",
-	         __func__, __LINE__, servername);
-	g_debug ("%s()[%u]   username = %s",
-	         __func__, __LINE__, username);
+	g_debug ("%s()[%u] servername = %s", __func__, __LINE__, servername);
+	g_debug ("%s()[%u]   username = %s", __func__, __LINE__, username);
 
 	/* Initialize backend cache */
 	if (priv->cal_cache != NULL) {
@@ -337,14 +321,13 @@ e_cal_backend_kolab_open (ECalBackendSync *backend,
 	sourcename = kolab_util_backend_get_relative_path_from_uri (priv->cal_uri);
 
 	/* Check whether we have a KolabMailAccess (KoMA) instance already */
-
 	user_at_server = g_strdup_printf ("%s@%s",
 	                                  username, servername);
-
-	ok = g_hash_table_lookup_extended (priv->koma_table,
+	ok = g_hash_table_lookup_extended (koma_objects,
 	                                   user_at_server,
 	                                   (gpointer *) &tmp_key,
 	                                   (gpointer *) &tmp_koma);
+
 	if (ok) {
 		gchar *passwd = NULL;
 
@@ -353,9 +336,6 @@ e_cal_backend_kolab_open (ECalBackendSync *backend,
 		 */
 		g_object_ref (tmp_koma);
 		priv->cal_koma = tmp_koma;
-		g_free (servername);
-		g_free (username);
-		g_free (user_at_server);
 		ksettings = kolab_mail_access_get_settings_handler (priv->cal_koma);
 		kolab_util_backend_prepare_settings (ksettings,
 		                                     NULL,
@@ -371,10 +351,9 @@ e_cal_backend_kolab_open (ECalBackendSync *backend,
 		passwd = g_strdup (kolab_settings_handler_get_char_field (ksettings,
 		                                                          KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_USER_PASSWORD,
 		                                                          NULL));
-		g_object_unref (ksettings);
 		if (passwd == NULL) {
-			e_cal_backend_notify_auth_required (E_CAL_BACKEND (self), TRUE, NULL);
-			return;
+			auth_required = TRUE;
+			goto exit;
 		}
 		g_free (passwd);
 
@@ -382,46 +361,34 @@ e_cal_backend_kolab_open (ECalBackendSync *backend,
 		                                                  online,
 		                                                  cancellable,
 		                                                  &tmp_err);
-		if (tmp_err != NULL) {
-			kolab_util_calendar_err_to_edb_err (error, tmp_err, __func__, __LINE__);
-			g_error_free (tmp_err);
-			return;
-		}
-		ok = cal_backend_kolab_notify_opened (self, &tmp_err);
-		if (! ok) {
-			kolab_util_calendar_err_to_edb_err (error, tmp_err, __func__, __LINE__);
-			g_error_free (tmp_err);
-		}
-		return;
+		goto exit;
 	}
 
 	/* Nope, we need to setup a new KoMA instance and a settings handler */
 
+	/* init subsystems (these are no-ops if already called before) */
+	kolab_util_glib_init ();
+	kolab_util_http_init ();
+	/* libcamel
+	 * Curl init may configure the underlying SSL lib,
+	 * but as far as SSL goes, we want Camel to rule here
+	 * TODO check whether Camel session needs to be initialized before or after libcurl.
+	 */
+	ok = kolab_util_camel_init (&tmp_err);
+	if (! ok)
+		goto exit;
+
 	/* Configure settings handler */
 	ksettings = KOLAB_SETTINGS_HANDLER (g_object_new (KOLAB_TYPE_SETTINGS_HANDLER, NULL));
 	ok = kolab_settings_handler_configure (ksettings,
 	                                       KOLAB_FOLDER_CONTEXT_CALENDAR,
 	                                       &tmp_err);
-	if (! ok) {
-		g_free (servername);
-		g_free (username);
-		g_free (user_at_server);
-		g_object_unref (ksettings);
-		kolab_util_calendar_err_to_edb_err (error, tmp_err, __func__, __LINE__);
-		g_error_free (tmp_err);
-		return;
-	}
+	if (! ok)
+		goto exit;
 
 	ok = kolab_settings_handler_bringup (ksettings, &tmp_err);
-	if (! ok) {
-		g_free (servername);
-		g_free (username);
-		g_free (user_at_server);
-		g_object_unref (ksettings);
-		kolab_util_calendar_err_to_edb_err (error, tmp_err, __func__, __LINE__);
-		g_error_free (tmp_err);
-		return;
-	}
+	if (! ok)
+		goto exit;
 
 	kolab_util_backend_prepare_settings (ksettings,
 	                                     esource,
@@ -431,36 +398,27 @@ e_cal_backend_kolab_open (ECalBackendSync *backend,
 	                                     sourcename,
 	                                     &sync_value);
 
-	g_free (servername);
-	g_free (username);
-	g_free (sourcename);
-
+	/* create new KolabMailAccess instance */
 	priv->cal_koma = KOLAB_MAIL_ACCESS (g_object_new (KOLAB_TYPE_MAIL_ACCESS, NULL));
 	g_object_add_toggle_ref (G_OBJECT (priv->cal_koma),
 	                         kolab_util_backend_koma_table_cleanup_cb,
-	                         priv->koma_table);
-	g_hash_table_insert (priv->koma_table,
+	                         koma_objects);
+	g_hash_table_insert (koma_objects,
 	                     user_at_server,
 	                     priv->cal_koma);
 
+	/* configure and bring up KolabMailAccess instance */
 	ok = kolab_mail_access_configure (priv->cal_koma,
 	                                  ksettings,
 	                                  &tmp_err);
-	g_object_unref (ksettings);
-	if (! ok) {
-		kolab_util_calendar_err_to_edb_err (error, tmp_err, __func__, __LINE__);
-		g_error_free (tmp_err);
-		return;
-	}
+	if (! ok)
+		goto exit;
 
 	ok = kolab_mail_access_bringup (priv->cal_koma,
 	                                cancellable,
 	                                &tmp_err);
-	if (! ok) {
-		kolab_util_calendar_err_to_edb_err (error, tmp_err, __func__, __LINE__);
-		g_error_free (tmp_err);
-		return;
-	}
+	if (! ok)
+		goto exit;
 
 	/* TODO this is sort of hackish, we cannot be sure that
 	 *      'user@server' is truly the Kolab email address
@@ -475,7 +433,43 @@ e_cal_backend_kolab_open (ECalBackendSync *backend,
 	 * (password in our case) anyway, so we can just request
 	 * it once the book gets opened.
 	 */
-	e_cal_backend_notify_auth_required (E_CAL_BACKEND (self), TRUE, NULL);
+	auth_required = TRUE;
+
+ exit:
+
+	if (ksettings != NULL)
+		g_object_unref (ksettings);
+
+	g_mutex_unlock (koma_objects_lock);
+
+	if (c_url != NULL)
+		camel_url_free (c_url);
+	if (servername != NULL)
+		g_free (servername);
+	if (username != NULL)
+		g_free (username);
+	if (user_at_server != NULL)
+		g_free (user_at_server);
+
+	/* do we have an error set? if so, propagate and return */
+	if (tmp_err != NULL) {
+		kolab_util_calendar_err_to_edb_err (error, tmp_err, __func__, __LINE__);
+		g_error_free (tmp_err);
+		return;
+	}
+
+	/* do we need authentication data? if so, notify and return */
+	if (auth_required) {
+		e_cal_backend_notify_auth_required (E_CAL_BACKEND (self), TRUE, NULL);
+		return;
+	}
+
+	/* all good, notify that we're open for business */
+	ok = cal_backend_kolab_notify_opened (self, &tmp_err);
+	if (! ok) {
+		kolab_util_calendar_err_to_edb_err (error, tmp_err, __func__, __LINE__);
+		g_error_free (tmp_err);
+	}
 }
 
 static void
@@ -1737,7 +1731,6 @@ e_cal_backend_kolab_start_view (ECalBackend *backend,
 {
 	ECalBackendKolab *self = NULL;
 	ECalBackendKolabPrivate *priv = NULL;
-	EDataCalView *cv = NULL;
 	GList *uid_list = NULL;
 	GList *it = NULL;
 	GSList *iCal_objects = NULL;
@@ -1753,17 +1746,9 @@ e_cal_backend_kolab_start_view (ECalBackend *backend,
 	self = E_CAL_BACKEND_KOLAB (backend);
 	priv = E_CAL_BACKEND_KOLAB_PRIVATE (self);
 
-	cv = g_hash_table_lookup (priv->active_cal_views, view);
-	if (cv != NULL) {
-		g_warning ("%s()[%u] cal view already active!",
-		           __func__, __LINE__);
-		return;
-	}
+	g_mutex_lock (active_cal_views_lock);
 
-	g_object_ref (view);
-	g_hash_table_insert (priv->active_cal_views,
-	                     view,
-	                     NULL);
+	g_object_ref (view); /* unref()d in stop_view() */
 
 	query = e_data_cal_view_get_text (view);
 
@@ -1815,7 +1800,21 @@ e_cal_backend_kolab_start_view (ECalBackend *backend,
 
 	e_data_cal_view_notify_objects_added (view,
 	                                      iCal_objects);
+
  exit:
+
+	if (tmp_err != NULL) {
+		kolab_util_calendar_err_to_edb_err (&view_err, tmp_err, __func__, __LINE__);
+		g_error_free (tmp_err);
+	}
+
+	e_data_cal_view_notify_complete (view, view_err);
+
+	g_mutex_unlock (active_cal_views_lock);
+
+	if (view_err != NULL)
+		g_error_free (view_err);
+
 	if (iCal_objects != NULL) {
 		g_slist_foreach (iCal_objects, (GFunc) g_free, NULL);
 		g_slist_free (iCal_objects);
@@ -1825,14 +1824,6 @@ e_cal_backend_kolab_start_view (ECalBackend *backend,
 	if (uid_list != NULL)
 		kolab_util_glib_glist_free (uid_list);
 
-	if (tmp_err != NULL) {
-		kolab_util_calendar_err_to_edb_err (&view_err, tmp_err, __func__, __LINE__);
-		g_error_free (tmp_err);
-	}
-
-	e_data_cal_view_notify_complete (view, view_err);
-	if (view_err != NULL)
-		g_error_free (view_err);
 }
 
 static void
@@ -1841,7 +1832,6 @@ e_cal_backend_kolab_stop_view (ECalBackend *backend,
 {
 	ECalBackendKolab *self = NULL;
 	ECalBackendKolabPrivate *priv = NULL;
-	EDataCalView *cv = NULL;
 
 	g_return_if_fail (E_IS_CAL_BACKEND_KOLAB (backend));
 	g_return_if_fail (E_IS_DATA_CAL_VIEW (view));
@@ -1849,15 +1839,13 @@ e_cal_backend_kolab_stop_view (ECalBackend *backend,
 	self = E_CAL_BACKEND_KOLAB (backend);
 	priv = E_CAL_BACKEND_KOLAB_PRIVATE (self);
 
-	cv = g_hash_table_lookup (priv->active_cal_views, view);
-	if (cv == NULL) {
-		g_warning ("%s()[%u] cal view already stopped!",
-		           __func__, __LINE__);
-		return;
-	}
+	g_mutex_lock (active_cal_views_lock);
 
 	e_data_cal_view_notify_complete (view, NULL);
-	g_hash_table_remove (priv->active_cal_views, view);
+
+	g_object_unref (view); /* ref()d in start_view() */
+
+	g_mutex_unlock (active_cal_views_lock);
 }
 
 static void
@@ -1887,14 +1875,35 @@ e_cal_backend_kolab_init (ECalBackendKolab *backend)
 	ECalBackendKolab *self = E_CAL_BACKEND_KOLAB (backend);
 	ECalBackendKolabPrivate *priv = E_CAL_BACKEND_KOLAB_PRIVATE (self);
 
-	g_debug ("%s()[%u] called.", __func__, __LINE__);
-
+	/* We're inside a thread, and each of our siblings
+	 * (other configured ECalBackendKolab instances) is
+	 * so as well. Depending on how the threads are
+	 * started, checking for lock existence (and creating
+	 * one if it is not) in *this* place can still be
+	 * racy. Let's hope the threads are started one
+	 * at a time, in which case this solution will work.
+	 * The whole handling of living KolabMailAccess
+	 * instances would better be moved to a separate
+	 * library...
+	 *
+	 */
+	if (koma_objects_lock == NULL)
+		koma_objects_lock = g_mutex_new ();
+	g_mutex_lock (koma_objects_lock);
 	if (koma_objects == NULL)
-		koma_objects = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-	else
-		g_hash_table_ref (koma_objects);
+		koma_objects = g_hash_table_new_full (g_direct_hash,
+		                                      g_direct_equal,
+		                                      g_free,
+		                                      g_object_unref);
+	g_mutex_unlock (koma_objects_lock);
+	/* active cal views are shared in the same way,
+	 * so we do the locking for them analogous
+	 * to the KolabMailAccess instances
+	 */
+	if (active_cal_views_lock == NULL)
+		active_cal_views_lock = g_mutex_new ();
 
-	priv->koma_table = koma_objects;
+	g_debug ("%s()[%u] called.", __func__, __LINE__);
 
 	priv->cal_mode = 0;
 	priv->cal_koma = NULL;
@@ -1904,10 +1913,6 @@ e_cal_backend_kolab_init (ECalBackendKolab *backend)
 	priv->source_type = 0;
 	priv->cal_uri = NULL;
 	priv->mode_switch_err = NULL;
-	priv->active_cal_views = g_hash_table_new_full (g_direct_hash,
-	                                                g_direct_equal,
-	                                                g_object_unref,
-	                                                NULL);
 
 	g_signal_connect (E_BACKEND (backend), "notify::online", G_CALLBACK (cal_backend_kolab_signal_online_cb), NULL);
 } /* e_cal_backend_kolab_init () */
@@ -1926,10 +1931,6 @@ e_cal_backend_kolab_dispose (GObject *object)
 			g_object_unref (priv->cal_koma);
 			priv->cal_koma = NULL;
 		}
-		if (priv->koma_table != NULL) {
-			g_hash_table_unref (priv->koma_table);
-			priv->koma_table = NULL;
-		}
 #endif
 	if (priv->default_zone != NULL) {
 		g_object_unref (priv->default_zone);
@@ -1939,10 +1940,6 @@ e_cal_backend_kolab_dispose (GObject *object)
 		g_object_unref (priv->cal_cache);
 		priv->cal_cache = NULL;
 	}
-	if (priv->active_cal_views != NULL) {
-		g_hash_table_destroy (priv->active_cal_views);
-		priv->active_cal_views = NULL;
-	}
 
 	G_OBJECT_CLASS (e_cal_backend_kolab_parent_class)->dispose (object);
 } /* e_cal_backend_kolab_dispose () */
