@@ -31,6 +31,7 @@
 
 #include <glib/gi18n-lib.h>
 
+#include <libedataserver/e-data-server-util.h>
 #include <libekolabutil/kolab-util-glib.h>
 
 #include "kolab-util-backend.h"
@@ -67,14 +68,18 @@ typedef gboolean (*KolabSettingsHandlerGetFunc) (KolabSettingsHandler*, GError**
 
 static gboolean kolab_settings_handler_char_get_func_camel_data_dir (KolabSettingsHandler*, GError**);
 static gboolean kolab_settings_handler_char_set_func_camel_data_dir (KolabSettingsHandler*, gchar*, GError**);
+static gboolean kolab_settings_handler_char_get_func_camel_cache_dir (KolabSettingsHandler*, GError**);
+static gboolean kolab_settings_handler_char_set_func_camel_cache_dir (KolabSettingsHandler*, gchar*, GError**);
 static gboolean kolab_settings_handler_char_get_func_camel_config_dir (KolabSettingsHandler*, GError**);
 static gboolean kolab_settings_handler_char_set_func_camel_config_dir (KolabSettingsHandler*, gchar*, GError**);
 static gboolean kolab_settings_handler_uint_set_func_folder_context (KolabSettingsHandler*, guint value, GError**);
 
 static KolabSettingsHandlerGetFunc _kolab_settings_handler_char_get_funcs[] = {
 	kolab_settings_handler_char_get_func_camel_data_dir, /* KOLAB_SETTINGS_HANDLER_CHAR_FIELD_CAMEL_DATA_DIR */
+	kolab_settings_handler_char_get_func_camel_cache_dir, /* KOLAB_SETTINGS_HANDLER_CHAR_FIELD_CAMEL_CACHE_DIR */
 	kolab_settings_handler_char_get_func_camel_config_dir, /* KOLAB_SETTINGS_HANDLER_CHAR_FIELD_CAMEL_CONFIG_DIR */
 	NULL, /* KOLAB_SETTINGS_HANDLER_CHAR_FIELD_CAMEL_ACCOUNT_DIR */
+	NULL, /* KOLAB_SETTINGS_HANDLER_CHAR_FIELD_ESOURCE_UID */
 	NULL, /* KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_SERVER_NAME */
 	NULL, /* KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_USER_NAME */
 	NULL, /* KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_USER_PASSWORD */
@@ -84,8 +89,10 @@ static KolabSettingsHandlerGetFunc _kolab_settings_handler_char_get_funcs[] = {
 
 static KolabSettingsHandlerCharSetFunc _kolab_settings_handler_char_set_funcs[] = {
 	kolab_settings_handler_char_set_func_camel_data_dir, /* KOLAB_SETTINGS_HANDLER_CHAR_FIELD_CAMEL_DATA_DIR */
+	kolab_settings_handler_char_set_func_camel_cache_dir, /* KOLAB_SETTINGS_HANDLER_CHAR_FIELD_CAMEL_CACHE_DIR */
 	kolab_settings_handler_char_set_func_camel_config_dir, /* KOLAB_SETTINGS_HANDLER_CHAR_FIELD_CAMEL_CONFIG_DIR */
 	NULL, /* KOLAB_SETTINGS_HANDLER_CHAR_FIELD_CAMEL_ACCOUNT_DIR */
+	NULL, /* KOLAB_SETTINGS_HANDLER_CHAR_FIELD_ESOURCE_UID */
 	NULL, /* KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_SERVER_NAME */
 	NULL, /* KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_USER_NAME */
 	NULL, /* KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_USER_PASSWORD */
@@ -140,13 +147,9 @@ kolab_settings_handler_char_get_func_camel_data_dir (KolabSettingsHandler *self,
 		g_assert_not_reached ();
 	}
 
-	/* FIXME implement getting of the user data dir */
-	g_error ("%s:  FIXME implement getting of the user data dir", __func__);
-#if 0
 	data_dir = g_build_filename (e_get_user_data_dir (),
 	                             backend_dir,
 	                             NULL);
-#endif
 
 	g_free (backend_dir);
 
@@ -164,6 +167,61 @@ kolab_settings_handler_char_get_func_camel_data_dir (KolabSettingsHandler *self,
 	return TRUE;
 }
 
+static gboolean
+kolab_settings_handler_char_get_func_camel_cache_dir (KolabSettingsHandler *self, GError **err)
+{
+	KolabSettingsHandlerPrivate *priv = NULL;
+	KolabFolderContextID context = KOLAB_FOLDER_CONTEXT_INVAL;
+	gchar *cache_dir = NULL;
+	gchar *priv_cache_dir = NULL;
+	gchar *backend_dir = NULL;
+
+	g_assert (KOLAB_IS_SETTINGS_HANDLER (self));
+	g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
+
+	priv = KOLAB_SETTINGS_HANDLER_PRIVATE (self);
+	context = priv->sdata_uint[KOLAB_SETTINGS_HANDLER_UINT_FIELD_FOLDER_CONTEXT];
+
+	priv_cache_dir = priv->sdata_char[KOLAB_SETTINGS_HANDLER_CHAR_FIELD_CAMEL_CACHE_DIR];
+	if (priv_cache_dir != NULL)
+		return TRUE;
+
+	/* TODO elaborate on directory creation */
+
+	switch (context) {
+	case KOLAB_FOLDER_CONTEXT_EMAIL:
+		backend_dir = g_strdup ("mail");
+		break;
+	case KOLAB_FOLDER_CONTEXT_CONTACT:
+		backend_dir = g_strdup ("addressbook");
+		break;
+	case KOLAB_FOLDER_CONTEXT_CALENDAR:
+		backend_dir = g_strdup ("calendar");
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+
+	cache_dir = g_build_filename (e_get_user_cache_dir (),
+	                              backend_dir,
+	                              NULL);
+
+	g_free (backend_dir);
+
+	if (g_mkdir_with_parents (cache_dir, 0700) != 0) {
+		g_set_error (err,
+		             G_FILE_ERROR,
+		             g_file_error_from_errno (errno),
+		             _("Could not create directory '%s' ('%s')"),
+		             cache_dir, g_strerror (errno));
+		g_free (cache_dir);
+		return FALSE;
+	}
+
+	priv->sdata_char[KOLAB_SETTINGS_HANDLER_CHAR_FIELD_CAMEL_CACHE_DIR] = cache_dir;
+	return TRUE;
+}
+
 static gboolean
 kolab_settings_handler_char_set_func_camel_data_dir (KolabSettingsHandler *self, gchar *value, GError **err)
 {
@@ -176,6 +234,19 @@ kolab_settings_handler_char_set_func_camel_data_dir (KolabSettingsHandler *self,
 	return FALSE;
 }
 
+static gboolean
+kolab_settings_handler_char_set_func_camel_cache_dir (KolabSettingsHandler *self, gchar *value, GError **err)
+{
+	(void)self;
+	(void)value;
+	/* FIXME mark as a translatable string */
+	g_set_error (err,
+	             KOLAB_BACKEND_ERROR,
+	             KOLAB_BACKEND_ERROR_GENERIC,
+	             "Cannot set Camel cache directory, it is a read-only resource");
+	return FALSE;
+}
+
 static gboolean
 kolab_settings_handler_char_get_func_camel_config_dir (KolabSettingsHandler *self, GError **err)
 {
