@@ -36,8 +36,8 @@
 #include <libedataserver/e-source.h>
 
 #include <libekolabutil/camel-system-headers.h>
+#include <libekolabutil/kolab-util-camel.h>
 
-#include "kolab-types.h"
 #include "kolab-mail-access.h"
 
 #include "kolab-util-backend.h"
@@ -123,24 +123,17 @@ kolab_util_backend_foldername_new_from_sourcename (const gchar *sourcename,
 gchar *
 kolab_util_backend_get_relative_path_from_uri (const gchar *uri)
 {
-	(void)uri;
-	/* FIXME implement me */
-	g_error ("%s: FIXME implement me", __func__);
-	return NULL;
-#if 0
 	CamelURL *c_url = NULL;
-	CamelException *cex = NULL;
 	gchar *tmp = NULL;
 	gchar *path = NULL;
+	GError *tmp_err = NULL;
 
-	cex = camel_exception_new ();
-	c_url = camel_url_new (uri, cex);
+	c_url = camel_url_new (uri, &tmp_err);
 	if (c_url == NULL) {
-		g_warning ("%s()[%u] error: %s", __func__, __LINE__, camel_exception_get_description (cex));
-		camel_exception_free (cex);
+		g_warning ("%s()[%u] error: %s", __func__, __LINE__, tmp_err->message);
+		g_error_free (tmp_err);
 		return NULL;
 	}
-	camel_exception_free (cex);
 	tmp = g_strdup (c_url->path);
 	camel_url_free (c_url);
 	if (tmp[0] == '/') {
@@ -152,7 +145,6 @@ kolab_util_backend_get_relative_path_from_uri (const gchar *uri)
 	}
 
 	return path;
-#endif
 } /* kolab_util_backend_get_relative_path_from_uri () */
 
 static gint
@@ -184,7 +176,10 @@ KolabSyncStrategyID
 kolab_util_misc_sync_value_from_property (const gchar *sync_prop)
 {
 	KolabSyncStrategyID sid = KOLAB_SYNC_STRATEGY_DEFAULT;
-	g_return_val_if_fail (sync_prop != NULL, KOLAB_SYNC_STRATEGY_DEFAULT);
+
+	if (sync_prop == NULL)
+		return KOLAB_SYNC_STRATEGY_DEFAULT;
+
 	sid = kolab_util_misc_generic_integer_from_property (sync_prop,
 	                                                     KOLAB_SYNC_STRATEGY_DEFAULT);
 	return sid;
@@ -194,7 +189,10 @@ KolabTLSVariantID
 kolab_util_misc_tls_variant_from_property (const gchar *tls_variant)
 {
 	KolabTLSVariantID tvid = KOLAB_TLS_VARIANT_DEFAULT;
-	g_return_val_if_fail (tls_variant != NULL, KOLAB_TLS_VARIANT_DEFAULT);
+
+	if (tls_variant == NULL)
+		return KOLAB_TLS_VARIANT_DEFAULT;
+
 	tvid = kolab_util_misc_generic_integer_from_property (tls_variant,
 	                                                      KOLAB_TLS_VARIANT_DEFAULT);
 	return tvid;
@@ -204,7 +202,10 @@ KolabReqPkcs11
 kolab_util_misc_req_pkcs11_from_property (const gchar *req_pkcs11_prop)
 {
 	KolabReqPkcs11 pkreq = KOLAB_PKCS11_INFRASTRUCTURE_DEFAULT;
-	g_return_val_if_fail (req_pkcs11_prop != NULL, KOLAB_PKCS11_INFRASTRUCTURE_DEFAULT);
+
+	if (req_pkcs11_prop == NULL)
+		return KOLAB_PKCS11_INFRASTRUCTURE_DEFAULT;
+
 	pkreq = kolab_util_misc_generic_integer_from_property (req_pkcs11_prop,
 	                                                       KOLAB_PKCS11_INFRASTRUCTURE_DEFAULT);
 	return pkreq;
@@ -292,6 +293,16 @@ kolab_util_backend_prepare_settings (KolabSettingsHandler *ksettings,
 	GError *error = NULL;
 	g_debug ("%s()[%u] called.", __func__, __LINE__);
 	if (esource != NULL) {
+		prop_str = e_source_peek_uid (esource);
+		(void)kolab_settings_handler_set_char_field (ksettings,
+		                                             KOLAB_SETTINGS_HANDLER_CHAR_FIELD_ESOURCE_UID,
+		                                             g_strdup (prop_str),
+		                                             &error);
+		if (error != NULL) {
+			g_warning ("%s()[%u] error: %s", __func__, __LINE__, error->message);
+			g_error_free (error);
+			error = NULL;
+		}
 		prop_str = e_source_get_uri (esource);
 		(void)kolab_settings_handler_set_char_field (ksettings,
 		                                             KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_URI,
@@ -438,3 +449,228 @@ kolab_util_backend_get_tls_variant_desc (KolabTLSVariantID id)
 
 	return gettext (kolab_tls_variant_desc[id]);
 }
+
+CamelURL*
+kolab_util_backend_camel_url_new_from_settings (KolabSettingsHandler *ksettings,
+                                                GError **err)
+{
+	KolabTLSVariantID tlsvariant = KOLAB_TLS_VARIANT_NONE;
+	CamelURL *camel_url = NULL;
+	const gchar *variant_str = NULL;
+	const gchar *tmp_str = NULL;
+	GError *tmp_err = NULL;
+
+	g_assert (KOLAB_IS_SETTINGS_HANDLER (ksettings));
+	g_return_val_if_fail (err == NULL || *err == NULL, NULL);
+
+	/* create url string for session getter */
+	camel_url = camel_url_new (KOLAB_CAMELURL_DUMMY_URL, NULL);
+	camel_url_set_protocol (camel_url, KOLAB_CAMEL_PROVIDER_PROTOCOL);
+	tmp_str = kolab_settings_handler_get_char_field (ksettings,
+	                                                 KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_SERVER_NAME,
+	                                                 &tmp_err);
+	if (tmp_err != NULL) {
+		camel_url_free (camel_url);
+		g_propagate_error (err, tmp_err);
+		return NULL;
+	}
+	if (tmp_str == NULL) {
+		camel_url_free (camel_url);
+		/* FIXME mark as a translatable string */
+		g_set_error (err,
+		             KOLAB_BACKEND_ERROR,
+		             KOLAB_BACKEND_ERROR_INTERNAL,
+		             "Internal inconsistency detected: Kolab server name not set");
+		return NULL;
+
+	}
+	camel_url_set_host (camel_url, tmp_str);
+	tmp_str = kolab_settings_handler_get_char_field (ksettings,
+	                                                 KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_USER_NAME,
+	                                                 &tmp_err);
+	if (tmp_err != NULL) {
+		camel_url_free (camel_url);
+		g_propagate_error (err, tmp_err);
+		return NULL;
+	}
+	if (tmp_str == NULL) {
+		camel_url_free (camel_url);
+		/* FIXME mark as a translatable string */
+		g_set_error (err,
+		             KOLAB_BACKEND_ERROR,
+		             KOLAB_BACKEND_ERROR_INTERNAL,
+		             "Internal inconsistency detected: Kolab User ID not set");
+		return NULL;
+
+	}
+	camel_url_set_user (camel_url, tmp_str);
+	tlsvariant = kolab_settings_handler_get_uint_field (ksettings,
+	                                                    KOLAB_SETTINGS_HANDLER_UINT_FIELD_TLS_VARIANT,
+	                                                    &tmp_err);
+	if (tmp_err != NULL) {
+		camel_url_free (camel_url);
+		g_propagate_error (err, tmp_err);
+		return NULL;
+	}
+
+	switch (tlsvariant) {
+	case KOLAB_TLS_VARIANT_SSL:
+		variant_str = KOLAB_CAMELURL_TLSVARIANT_SSL;
+		break;
+	case KOLAB_TLS_VARIANT_STARTTLS:
+		variant_str = KOLAB_CAMELURL_TLSVARIANT_STARTTLS;
+		break;
+	default:
+		variant_str = KOLAB_CAMELURL_TLSVARIANT_NONE;
+	}
+
+	camel_url_set_param (camel_url,
+	                     KOLAB_CAMELURL_PARAM_SSL,
+	                     variant_str);
+	return camel_url;
+}
+
+gchar*
+kolab_util_backend_account_uid_new_from_settings (KolabSettingsHandler *ksettings,
+                                                  GError **err)
+{
+	const gchar *user = NULL;
+	const gchar *server = NULL;
+	const gchar *esource_uid = NULL;
+	gchar *account_uid = NULL;
+	gchar *tmp_str = NULL;
+	GError *tmp_err = NULL;
+
+	g_assert (KOLAB_IS_SETTINGS_HANDLER (ksettings));
+	g_return_val_if_fail (err == NULL || *err == NULL, NULL);
+
+	esource_uid = kolab_settings_handler_get_char_field (ksettings,
+	                                                     KOLAB_SETTINGS_HANDLER_CHAR_FIELD_ESOURCE_UID,
+	                                                     &tmp_err);
+	if (tmp_err != NULL) {
+		g_warning ("%s: %s", __func__, tmp_err->message);
+		g_error_free (tmp_err);
+		tmp_err = NULL;
+	}
+
+	if (esource_uid != NULL) {
+		tmp_str = g_strdup (esource_uid);
+		goto done;
+	}
+
+	/* fallback in case we don't have an ESource UID set */
+
+	user = kolab_settings_handler_get_char_field (ksettings,
+	                                              KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_USER_NAME,
+	                                              &tmp_err);
+	if (tmp_err != NULL) {
+		g_propagate_error (err, tmp_err);
+		return NULL;
+	}
+	server = kolab_settings_handler_get_char_field (ksettings,
+	                                                KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_SERVER_NAME,
+	                                                &tmp_err);
+	if (tmp_err != NULL) {
+		g_propagate_error (err, tmp_err);
+		return NULL;
+	}
+
+	tmp_str = g_strconcat (user, "@", server, NULL);
+
+ done:
+
+	account_uid = g_strconcat (KOLAB_CAMEL_PROVIDER_PROTOCOL, "__", tmp_str, NULL);
+	g_free (tmp_str);
+
+	return account_uid;
+}
+
+KolabMailAccessOpmodeID
+kolab_util_backend_deploy_mode_by_koma (KolabMailAccess *koma,
+                                        KolabMailAccessOpmodeID koma_mode,
+                                        GCancellable *cancellable,
+                                        GError **error)
+{
+	KolabMailAccessOpmodeID tmp_mode = KOLAB_MAIL_ACCESS_OPMODE_INVAL;
+	GError *tmp_err = NULL;
+	gboolean ok = FALSE;
+
+	g_debug ("%s()[%u] called.", __func__, __LINE__);
+
+	if (koma != NULL) g_assert (KOLAB_IS_MAIL_ACCESS (koma));
+	g_assert (koma_mode < KOLAB_MAIL_ACCESS_LAST_OPMODE);
+	/* cancellable may be NULL */
+	g_return_val_if_fail (error == NULL || *error == NULL, KOLAB_MAIL_ACCESS_OPMODE_INVAL);
+
+	if (koma == NULL) {
+		g_debug ("%s()[%u] error: KolabMailAccess object not existent.",
+		         __func__, __LINE__);
+		/* FIXME set GError */
+		return KOLAB_MAIL_ACCESS_OPMODE_INVAL;
+	}
+
+	/* get current KolabMailAccess operational mode */
+	tmp_mode = kolab_mail_access_get_opmode (koma, &tmp_err);
+	if (tmp_err != NULL) {
+		g_propagate_error (error, tmp_err);
+		return KOLAB_MAIL_ACCESS_OPMODE_INVAL;
+	}
+	if (tmp_mode < KOLAB_MAIL_ACCESS_OPMODE_OFFLINE) {
+		g_debug ("%s()[%u] KolabMailAccess object not ready, deferring.",
+		         __func__, __LINE__);
+		return tmp_mode;
+	}
+
+	/* set new KolabMailAccess operational mode */
+	ok = kolab_mail_access_set_opmode (koma,
+	                                   koma_mode,
+	                                   cancellable,
+	                                   &tmp_err);
+	if (! ok) {
+		g_propagate_error (error, tmp_err);
+		return KOLAB_MAIL_ACCESS_OPMODE_INVAL;
+	}
+
+	return koma_mode;
+}
+
+gboolean
+kolab_util_backend_deploy_mode_by_backend (KolabMailAccess *koma,
+                                           gboolean online,
+                                           GCancellable *cancellable,
+                                           GError **error)
+{
+	KolabMailAccessOpmodeID koma_mode = KOLAB_MAIL_ACCESS_OPMODE_INVAL;
+	KolabMailAccessOpmodeID tmp_mode = KOLAB_MAIL_ACCESS_OPMODE_INVAL;
+	GError *tmp_err = NULL;
+
+	g_debug ("%s()[%u] called.", __func__, __LINE__);
+
+	if (koma != NULL) g_assert (KOLAB_IS_MAIL_ACCESS (koma));
+	g_assert (koma_mode < KOLAB_MAIL_ACCESS_LAST_OPMODE);
+	/* cancellable may be NULL */
+	g_return_val_if_fail (error == NULL || *error == NULL, KOLAB_MAIL_ACCESS_OPMODE_INVAL);
+
+	if (koma == NULL) {
+		g_debug ("%s()[%u] error: KolabMailAccess object not existent.",
+		         __func__, __LINE__);
+		/* FIXME set GError */
+		return KOLAB_MAIL_ACCESS_OPMODE_INVAL;
+	}
+
+	switch (online) {
+	case FALSE:
+		koma_mode = KOLAB_MAIL_ACCESS_OPMODE_OFFLINE;
+		break;
+	default:
+		koma_mode = KOLAB_MAIL_ACCESS_OPMODE_ONLINE;
+	}
+	tmp_mode = kolab_util_backend_deploy_mode_by_koma (koma,
+	                                                   koma_mode,
+	                                                   cancellable,
+	                                                   &tmp_err);
+	if (tmp_err != NULL)
+		g_propagate_error (error, tmp_err);
+
+	return tmp_mode == koma_mode;
+}
